
// ************* TinyFlea microcomputer framework ************
// ** For use with Fleatiny system board revision 1.2+ PCBs **
// **************** Kernel build # 0051 **********************

// *************** kernel_sd.c module *******************
// SD card interface module for fleatiny 
// Provides initialisation and raw sector read/write of MMC/SD/SDHC cards 
// whose sectors are 512 Bytes long (ie 2GB SD cards will not work without
// modification to the code contained in this module).
// 
// Main Functions: 
// ---------------
// Function #1: "unsigned char sd_init(void)"
// Usage:  return_error = sd_init(); 
// Where:  return_error = 0 if initialisation sequence OK

// Function #2: "unsigned char sd_iosector(unsigned long sector_address)"
// Usage:   sd_sector_read = 1; return_error = sd_iosector(1);
// Where:   'sd_sector_read' flag sets read/write mode, while 
//          return_error = 0 if transfer successful


//  Copyright(c) 2012 Valentin Angelovski

//  This file is part of my Fleatiny embedded framework. Please refer 
//  to <http://www.fleasystems.com/fleatiny.html> for further info.

//  Fleatiny embedded framework is free software: you can redistribute it and/or 
//  modify it under the terms of the GNU Lesser General Public License as published 
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  Fleatiny embedded framework is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.

//   You should have received a copy of the GNU Lesser General Public License
//   along with Fleatiny embedded framework.  If not, see <http://www.gnu.org/licenses/>.


//********** Kernel_SD module I/O usage **********
//__sbit __at (0x94) SD_CS		;//P1.4 
//__sbit __at (0x95) SD_MOSI 	;//P1.5 
//__sbit __at (0x96) SD_MISO 	;//P1.6
//__sbit __at (0x97) SD_CLK		;//P1.7 



void wait_100ms(void)
{


__asm	
MOV	_COUNT3,#0xFF
MOV	_COUNT1,#0xFF

wait100:
DJNZ 	_COUNT1, wait100
DJNZ 	_COUNT3, wait100
__endasm;


}

void get_crc7(void) {


zcrc=0;
for (za=0;za<5;za++) {
zData=sd_cmdpacket[za];
for (zi=0;zi<8;zi++) {
zcrc <<= 1;

if ((zData & 0x80)^(zcrc & 0x80))
zcrc ^=0x09;
zData <<= 1;
}
}
zcrc=(zcrc<<1)|1;
sd_cmdpacket[5] = zcrc;



}



void spi_write(unsigned char bytewt, unsigned char spispeed) 
{

if(spispeed == 0) COUNT1=1;
if(spispeed == 1) COUNT1=1;
if(spispeed == 2) COUNT1=1;
if(spispeed == 3) COUNT1=1;
	COUNT2=bytewt;

__asm	

MOV		A,_COUNT2


MOV		_COUNT3,#0x08
spiwtnxt:

CLR		_SD_CLK
RLC		A
MOV		_SD_MOSI,C

MOV		_COUNT2,_COUNT1
spiwp1:
DJNZ 	_COUNT2, spiwp1
nop
SETB	_SD_CLK	

MOV		_COUNT2,_COUNT1
spiwp4:
DJNZ 	_COUNT2, spiwp4

	
DJNZ	_COUNT3,spiwtnxt
CLR		_SD_CLK

__endasm;

}

unsigned char spi_read(char spispd) 
{

if(spispd == 0) COUNT1=1;
if(spispd == 1) COUNT1=1;
if(spispd == 2) COUNT1=1;
if(spispd == 3) COUNT1=1;

__asm	

CLR		_SD_CLK
;mov		C,_SD_MISO
;RLC		A

MOV		_COUNT3,#0x08


spirdnxt:

MOV		_COUNT2,_COUNT1
spirp1:
DJNZ 	_COUNT2, spirp1

SETB	_SD_CLK	

MOV		_COUNT2,_COUNT1
spirp3:
DJNZ 	_COUNT2, spirp3

nop
mov		C,_SD_MISO
RLC		A
CLR		_SD_CLK	
DJNZ	_COUNT3,spirdnxt

MOV		_COUNT3,A


__endasm;

return(COUNT3);

}


//	FAST SD CARD (OR SPI-IO) BLOCK READ ROUTINE - 
void fast_spi_rd(void)
{

__asm
	mov		dptr,#_sd_buffer
	mov		a,dph
	add		a,#0x02
	mov		_COUNT3,a
	mov		_COUNT2,dpl
	CLR		_SD_CLK
	
rd_sdbyte:	
	

;//	MOV		_COUNT1,#0x08


	
spifastrd1:

	SETB	_SD_CLK						
	mov		C,_SD_MISO
	CLR		_SD_CLK	
	RLC		A			
	SETB	_SD_CLK						
	mov		C,_SD_MISO
	CLR		_SD_CLK	
	RLC		A	
	SETB	_SD_CLK						
	mov		C,_SD_MISO
	CLR		_SD_CLK	
	RLC		A	
	SETB	_SD_CLK						
	mov		C,_SD_MISO
	CLR		_SD_CLK	
	RLC		A	
	SETB	_SD_CLK						
	mov		C,_SD_MISO
	CLR		_SD_CLK	
	RLC		A	
	SETB	_SD_CLK						
	mov		C,_SD_MISO
	CLR		_SD_CLK	
	RLC		A	
	SETB	_SD_CLK						
	mov		C,_SD_MISO
	CLR		_SD_CLK	
	RLC		A		
	SETB	_SD_CLK						
	mov		C,_SD_MISO
	CLR		_SD_CLK	
	RLC		A		
;//	DJNZ	_COUNT1,spifastrd1

	movx	@dptr,a
	inc		dptr

	mov		a,dph	
	cjne	a,_COUNT3,rd_sdbyte		;// Is 512 Bytes done yet?
	mov		a,dpl	
	cjne	a,_COUNT2,rd_sdbyte		;// Is 512 Bytes done yet?
	
CLR		_SD_CLK
	
__endasm;

}


//	FAST SD CARD (OR SPI-IO) BLOCK READ ROUTINE - 
void fast_spi_wr(void)
{

spi_write(0xFF,1);
spi_write(0xFE,1);

__asm
	mov		dptr,#_sd_buffer
	mov		a,dph
	add		a,#0x02
	mov		_COUNT3,a
	mov		_COUNT2,dpl
	CLR		_SD_CLK
	
wr_sdbyte:	
;//	MOV		_COUNT1,#0x08

	movx	a,@dptr
	inc		dptr
	
spifastwr1:
	RLC		A
	CLR		_SD_CLK
	MOV		_SD_MOSI,C					
	SETB	_SD_CLK	
	RLC		A
	CLR		_SD_CLK
	MOV		_SD_MOSI,C					
	SETB	_SD_CLK	
	RLC		A
	CLR		_SD_CLK
	MOV		_SD_MOSI,C					
	SETB	_SD_CLK	
	RLC		A
	CLR		_SD_CLK
	MOV		_SD_MOSI,C					
	SETB	_SD_CLK	
	RLC		A
	CLR		_SD_CLK
	MOV		_SD_MOSI,C					
	SETB	_SD_CLK	
	RLC		A
	CLR		_SD_CLK
	MOV		_SD_MOSI,C					
	SETB	_SD_CLK	
	RLC		A
	CLR		_SD_CLK
	MOV		_SD_MOSI,C					
	SETB	_SD_CLK	
	RLC		A
	CLR		_SD_CLK
	MOV		_SD_MOSI,C					
	SETB	_SD_CLK	
;//	DJNZ	_COUNT1,spifastwr1



	mov		a,dph	
	cjne	a,_COUNT3,wr_sdbyte		;// Is 512 Bytes done yet?
	mov		a,dpl	
	cjne	a,_COUNT2,wr_sdbyte		;// Is 512 Bytes done yet?
	
CLR		_SD_CLK
	
__endasm;

}


void sd_command(unsigned char sdcmd, unsigned long argument, unsigned char spispeed)
{	

//	stop_here:
//	if(argument == 0x00004040) goto stop_here;

	sd_cmdpacket[0] = sdcmd | 0x40;	// ALWAYS SET BIT#6 OF THE COMMAND BYTE!

	sd_cmdpacket[1] = (argument >> 24) & 0xff; 
	sd_cmdpacket[2] = (argument >> 16) & 0xff; 
	sd_cmdpacket[3] = (argument >> 8) & 0xff; 
	sd_cmdpacket[4] = (argument >> 0) & 0xff; 

	get_crc7();

	spi_write(sd_cmdpacket[0],spispeed);
	spi_write(sd_cmdpacket[1],spispeed);	
	spi_write(sd_cmdpacket[2],spispeed);
	spi_write(sd_cmdpacket[3],spispeed);
	spi_write(sd_cmdpacket[4],spispeed);
	spi_write(sd_cmdpacket[5],spispeed);

}

void enable_spi(void)
{
__asm
		CLR		_SD_CLK
		SETB	_SD_MOSI
__endasm;

}

void disable_spi(void)
{
__asm
		CLR		_SD_CLK
		SETB	_SD_MOSI
__endasm;		

}


void sd_getresponse(void)
{
	temp1 = spi_read(1);			// CRC Bytes
	sd_respons1 = spi_read(1);		// 
	
	sd_respons2 = spi_read(1);	    // Data response token
	sd_respons3 = spi_read(1);	
	sd_respons4 = spi_read(1);	
	sd_respons5 = spi_read(1);	
	sd_respons5 = spi_read(1);
}

void millisecs_pause(unsigned char waitmsecs)
{

COUNT3 = waitmsecs;

__asm	
MOV		_COUNT1,#10
INC		_COUNT3
waits_loop:
DJNZ 	_COUNT1, waits_loop
DJNZ 	_COUNT3, waits_loop


__endasm;

}

void select_sd(void)
{
    enable_spi();
	millisecs_pause(1);
  	SD_CS=0; 
}

void unselect_sd(void)
{
	SD_CS=1; 
	millisecs_pause(1);  
	disable_spi();	
}


unsigned char sd_init(void) 
{
	
	
retry_cmd0:
   
   // Global CardType - b0:MMC, b1:SDv1, b2:SDv2

   
   /* disable SPI chip select... */
    enable_spi();
//    delay(1);
   	SD_CS=1;   
	wait_100ms();	
//	delay(1);


   /* fill send data with all ones - 80 bits long to   */
   /* establish link with SD card this fulfills the    */
   /* 74 clock cycle requirement...  */
   
  	for(Count=0;Count<10;Count++) spi_write(0xFF,0);
	wait_100ms();
  	SD_CS=0; 	

	
	
	sd_command(0x00,0,0);
	
	/* Send Command0 (enter idle-mode) */	
	for(Count = 0;Count < 1000;Count++)
	{	
	sd_respons1 = spi_read(0);  // Get response
	if((sd_respons1 & 0xf7) == 1 )  goto end_cmd0;
	}

	unselect_sd();	
	return(1);	
	
end_cmd0:	
	COUNT1 = spi_read(0);		// Dummy read to add extra 8 clocks from host (i.e. us)


	
//	if(sd_respons1 == 0x01) goto end_cmd59;

//	SD_CS=1; 
//	millisecs_pause(1);  
//	disable_spi();		
//	return(sd_respons1);	
	
//end_cmd59:	



	// *************** CMD8 Check if SD card is 2.7-3.3V supply compatible ****************
	for(Count = 0;Count < 100;Count++)
	{	

	sd_command(8, 0x000001AA, 0);	//CMD8

	sd_getresponse();
	
	COUNT1 = spi_read(0);	
	
	if(sd_respons4 == 0x05) goto send_cmd1;	// Illegal command - MUST be sd version 1.xx!
	
	if((sd_respons4 == 0x01) && (sd_respons5 = 0xAA)) goto end_cmd8;
	}

	goto send_cmd1;
	
	/*
cmd8_bad:
cmd8_good = 0;
  	SD_CS=1; 
	millisecs_pause(100); 
	disable_spi();	
	return(3);	
	*/
	
end_cmd8:
	cmd8_good = 1;
	COUNT1 = spi_read(0);	

	// *************** CMD59 Enable command packet CRC read on the SD card ****************
//	sd_command(59,1,0);
//	COUNT1 = spi_read(0);		// Dummy read to add extra 8 clocks from host (i.e. us)
//	sd_respons1 = spi_read(0);  // Get response


	// *************** ACMD41-CMD55 Check if SD card is 3.3V compatible ****************
	for(Count = 0;Count < 1000;Count++)
	{		
		sd_command(55, 0, 0);   		// ACMD41  query if voltage range 3.2-3.3V ok to use?
	
		COUNT1 = spi_read(0);	
		sd_respons1 = spi_read(0);	
		COUNT1 = spi_read(0);
		
		sd_command(41, 0x40080000, 0);  //	 "  "   "  "  
		COUNT1 = spi_read(0);	
		sd_respons1 = spi_read(0);		
		COUNT1 = spi_read(0);		
		if(sd_respons1 == 0x00) goto end_cmd55;
	}
	
//	if((sd_respons4 != 0x01) && (sd_respons5 != 0xAA)) return(1);
	unselect_sd();		
	return(4);	
	
end_cmd55:



	// *************** CMD58 read CCS (Bit 30 in OCR) to determine if disk is SDHC ****************
	sd_command(58, 0, 0);   		
	
	sd_getresponse();
	
	if(sd_respons2 & 0x40) sd_hc = 1;


	
	// ************** CMD16 set block length ******************
	COUNT1 = spi_read(0);
	sd_command(16, 512, 0);   		// CMD16 set block length	
	COUNT1 = spi_read(0);	
	sd_respons1 = spi_read(0);		

	// ************** CMD7 Enter transfer mode ******************	
if(cmd8_good)
	{
	sd_command(7,0x00000000,1);	// Put SD card into transfer state
	COUNT1 = spi_read(0);	
	sd_respons1 = spi_read(0);		
	COUNT1 = spi_read(0);
	}
	
	unselect_sd();	
	return(0);	
	

// Test for MMC card	
send_cmd1:
  	SD_CS=0;
	
	/* Send Command1 (Attempt MMC enter idle-mode) */	
  	for(Count=0;Count<10000;Count++) 	
	{
		sd_command(0x01,0x00000000,0);
		COUNT1 = spi_read(0);		// Dummy read to add extra 8 clocks from host (i.e. us)				
		sd_respons1 = spi_read(0);  // Get response
		if(sd_respons1 == 0x00) goto end_cmd1;  // Command valid - MUST be MMC!		
	}
	unselect_sd();	
	return(3);	
	
	
end_cmd1:
	unselect_sd();	
	return(0);			
	
	
}

/*
unsigned char sd_status(void)
{
	select_sd();
	sd_command(13, 0, 0);   		
	
	sd_getresponse();
	
	unselect_sd();	
	
	if((sd_respons4 & 0x1E) != 0) return(sd_respons4);
	else return(0);
}
*/

unsigned char sd_iosector(unsigned long sector_address) 
{
   
 unsigned short i;
 //unsigned char mini_str[2];   
 
 //	mini_str[1]=0x00; // Pre null-terminate data dump string

   /* Enable software SPI and select SD card via /CS line */
	select_sd();	

	/* Send Command 17 (read single sector) */	
	if(sd_sector_read == 1) COUNT1 = 17;
	else COUNT1 = 24;
	
	sd_command(COUNT1,sector_address,1);	// Read single 512 block of data at FULL SPI rate (MHz, not kHz..)

	opcode = spi_read(1);				// Read in 1 dummy byte response delay to satisfy NCR condition

//		sd_respons1 = spi_read(0);			// Read in data token
	
	/* Check the data token value, and proceed with read if no errors */
	for(i = 0;i < 255;i++)	// Evaluate positive integer constant
	{
		sd_respons1 = spi_read(1);
	
		if(sd_sector_read == 1) COUNT2 = 0xFE;
		else COUNT2 = 0x00;
		if(sd_respons1 == COUNT2)	
		{
			goto go_for_sd_read;	
		}	
	}

	

wr_error:	
	unselect_sd();	
	return(1);	
	
go_for_sd_read:

	/* Read in sector contents from SD card */

	if(sd_sector_read == 1) fast_spi_rd();
	else fast_spi_wr(); 
	/* Read in CRC value (but do nothing with it for now,,) */
	sd_getresponse();
	

	
	if(sd_sector_read == 0) // Read Data response token while in write mode
	{
		if((sd_respons2 & 0x0F) != 0x05) 
		{ 			
			unselect_sd();
			return(2);
		}
		millisecs_pause(10);	

	for(i = 0;i < 10000;i++)	// Evaluate positive integer constant
	{		
		if(spi_read(0) != 0x00) goto sd_write_done;		// If SD card pull it's data out line high, write cycle is complete.
		millisecs_pause(1);
	}

	unselect_sd();	// If we get here, our write cycle failed
	return(3);		
	}
	
	unselect_sd();		
	return(0);	


sd_write_done:	
//	spi_write(0xFF,1);	
  	
	for(i=0;i<11;i++) spi_write(0xFF,1);	
	millisecs_pause(1);	
	unselect_sd();	
//	spi_write(0xFF,1);		
	return(0);		
	
}
