// ************* TinyFlea microcomputer framework ************
// ** For use with Fleatiny system board revision 1.2+ PCBs **
// ** or compatible 8051-based hardware clones              **
// **************** Kernel build # 0051 **********************

// *************** Fleatiny_vterm.c module *******************
// Provides basic video teletype functions to the application 
// programmer (ie. you! ;-) via the PS/2 keyboard and VGA output port


// Main Functions: 
// ---------------
// Function #1: "void tty_inchar(void)"
// Usage:  tty_inchar(); myvariable = newChar;
// Where:  After the above function is executed, fleatiny kernel variable 'newChar' should be equal
//         to the character entered on the PS/2 keyboard. 

// Function #2: "void tty_outchar(void)"
// Usage:   newChar = 'X'; tty_outchar();
// Where:   fleatiny kernel variable 'newChar' must be preloaded with a single ascii 
//			character prior to calling the tty_outchar function
         
		 
//  Copyright(c) 2012 Valentin Angelovski

//  This file is part of my Fleatiny embedded framework. Please refer 
//  to <http://www.fleasystems.com/fleatiny.html> for further info.

//  Fleatiny embedded framework is free software: you can redistribute it and/or 
//  modify it under the terms of the GNU Lesser General Public License as published 
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  Fleatiny embedded framework is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.

//   You should have received a copy of the GNU Lesser General Public License
//   along with Fleatiny embedded framework.  If not, see <http://www.gnu.org/licenses/>.


// *** TTY Character read (from PS/2 keyboard) function
void tty_inchar(void)
{
	wait_keypress:
		if(TL1 < 0xF0)
		{	
			if((GOT_KBRAW < 0x80) && (PS2_GOTK ==1))
				{			
				newchar = GOT_KBRAW;
				PS2_GOTK = 0;
			}
			else
			{
				goto wait_keypress;
				}
		}
		else
		{
		goto wait_keypress;
		}
}

// *** TTY Character write (to VGA) function
void tty_outchar(void)
{
		// We got a valid keypress if we get to here
				if(newchar == 8)	// Backspace key detected!				
				{
					if(cursor_xpos > 1) cursor_xpos--; 
					matrix_char[cursor_ypos][cursor_xpos] = ' ' + 10;	
					goto ttyoutchar_end; // Exit function
				}
		
				if(newchar == 13)	// Carriage return detected!
				{
					cursor_xpos=0;
					goto ttyoutchar_end; // Exit function
				}

				if(newchar == 10)	// Line feed detected!
				{
					goto go_to_newline; // Perform newline
				}				
				
				// Any other key..
				matrix_char[cursor_ypos][cursor_xpos] = (newchar & 0x7F) + 10; // Change the '+33' offset when using the UART video out method

				if(cursor_xpos < 31) 
				{
					cursor_xpos++;
				}
				else
				{
				go_to_newline:	
					cursor_xpos=0;
					if(cursor_ypos < 15) 
					{					
						cursor_ypos++;
					}
					else
					{
						for(dataptr=0;dataptr<480;dataptr++)
						{
							
							opcode = VRAM[dataptr+32];
							VRAM[dataptr] = opcode;
						}
						for(dataptr=480;dataptr<512;dataptr++)
						{
							VRAM[dataptr] = ' '+10;
						}						
						cursor_ypos = 15;
					}
				}
			ttyoutchar_end:

}




// *** low-level VGA text renderer assembly routine
void TEXTMODE(void)
{

__asm
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP


	MOV		_lcount,#0x20
	
	CLR		P1.1	
	
	Mov		DPTR,#JmpTbl	
	MOV		A,_burst1
	RL		A
	RL		A	
	INC		_burst1	
	ANL		_burst1,#0x07	
	
	Jmp		@A+Dptr
JmpTbl:
	LJMP	do_line1
	NOP
	LJMP	do_line2
	NOP
	LJMP	do_line3
	NOP
	LJMP	do_line4	
	NOP
	LJMP	do_line5
	NOP
	LJMP	do_line6
	NOP
	LJMP	do_line7
	NOP
	LJMP	do_line8

	
	
do_line1:	
	MOV  	DPL,_vpl
	MOV  	DPH,_vph

	
repeat_char1:
	MOVX 	A,@DPTR     
	INC     DPTR  	    
	MOVC	A,@A+PC     
	MOV		SBUF,A
	NOP
	NOP	    
	DJNZ	_lcount,repeat_char1 

	SETB	P1.1
  	RET	
	
charline1:
.db  255, 129, 129, 201, 247, 227, 247, 247, 0, 255, 255, 255, 255, 255, 1, 102, 254, 191, 231, 153, 1, 131, 255, 231
.db  231, 231, 255, 255, 255, 255, 255, 255, 255, 243, 201, 201, 243, 255, 227, 249, 231, 249, 255, 255, 255, 255, 255
.db  159, 193, 243, 225, 225, 199, 192, 227, 192, 225, 225, 255, 255, 231, 255, 249, 225, 193, 243, 192, 195, 224, 128
.db  128, 195, 204, 225, 135, 152, 240, 156, 156, 227, 192, 225, 192, 225, 192, 204, 204, 156, 156, 204, 128, 225, 252
.db  225, 247, 255, 243, 255, 248, 255, 199, 255, 227, 255, 248, 243, 207, 248, 241, 255, 255, 255, 255, 255, 255, 255
.db  247, 255, 255, 255, 255, 255, 255, 199, 231, 248, 145, 255, 225, 255, 199, 129, 204, 248, 243, 255, 129, 204, 248
.db  204, 193, 248, 156, 243, 199, 255, 131, 225, 255, 255, 225, 255, 255, 60, 204, 231, 227, 204, 224, 143, 199, 227
.db  255, 255, 255, 192, 195, 227, 243, 255, 255, 60, 60, 231, 255, 255, 187, 85, 36, 231, 231, 231, 147, 255, 255, 147
.db  147, 255, 147, 147, 231, 255, 231, 231, 255, 231, 255, 231, 231, 147, 147, 255, 147, 255, 147, 255, 147, 231, 147
.db  255, 255, 147, 231, 255, 255, 147, 231, 231, 255, 0, 255, 240, 15, 0, 255, 255, 255, 255, 192, 255, 255, 255, 192
.db  227, 227, 199, 255, 159, 227, 225, 255, 243, 249, 231, 143, 231, 243, 255, 227, 255, 255, 15, 225, 241, 255, 255

do_line2:	
	MOV  	DPL,_vpl
	MOV  	DPH,_vph
repeat_char2:
	MOVX 	A,@DPTR     
	INC     DPTR  	    
	MOVC	A,@A+PC     
	MOV		SBUF,A
	NOP
	NOP	    
	DJNZ	_lcount,repeat_char2


	SETB	P1.1	
  	RET	


charline2:
.db 255,126,0,128,227,193,247,247,0,255,255,255,255,255,57,165,248,143,195,153,36,57,255,195,195,231,231,243,243,243,243
.db 243,255,225,201,201,193,156,201,249,243,243,153,243,255,255,255,207,156,241,204,204,195,252,249,204,204,204,243,243,243
.db 255,243,204,156,225,153,153,201,185,185,153,204,243,207,153,249,136,152,201,153,204,153,204,210,204,204,156,156,204,156
.db 249,249,231,227,255,243,255,249,255,207,255,201,255,249,255,255,249,243,255,255,255,255,255,255,255,243,255,255,255,255
.db 255,255,243,231,243,196,247,204,204,255,60,255,255,243,255,60,255,255,255,156,255,227,243,255,255,201,204,204,248,204,248
.db 204,231,255,231,201,204,204,39,255,255,199,199,224,255,201,201,255,255,255,156,156,231,51,204,238,170,17,231,231,231,147,255
.db 255,147,147,255,147,147,231,255,231,231,255,231,255,231,231,147,147,255,147,255,147,255,147,231,147,255,255,147,231,255,255,147
.db 231,231,255,0,255,240,15,0,255,225,192,128,204,255,153,145,243,201,201,243,255,207,249,204,192,243,243,243,39,231,243,145,201
.db 255,255,207,201,231,255,255

do_line3:	
	MOV  	DPL,_vpl
	MOV  	DPH,_vph
repeat_char3:
	MOVX 	A,@DPTR     
	INC     DPTR  	    
	MOVC	A,@A+PC     
	MOV		SBUF,A
	NOP
	NOP    
	DJNZ	_lcount,repeat_char3
	SETB	P1.1	
  	RET	


	
charline3:
.db 255,90,36,128,193,227,227,227,24,255,255,255,255,255,1,195,224,131,129,153,36,227,255,129,129,231,207,249,249,249,249,249
.db 255,225,201,128,252,204,227,252,249,231,195,243,255,255,255,231,140,243,207,207,201,224,252,207,204,204,243,243,249,192,231,207
.db 132,204,153,252,153,233,233,252,204,243,207,201,249,128,144,156,153,204,153,248,243,204,204,156,201,204,206,249,243,231,201,255
.db 231,225,249,225,207,225,249,145,201,241,207,153,243,204,224,225,196,145,196,193,193,204,204,156,156,204,192,243,231,243,255,227
.db 252,255,225,195,225,225,225,225,195,225,225,241,227,241,201,255,192,1,204,255,255,255,255,255,255,195,204,129,217,225,204,231,225
.db 241,255,255,255,204,201,201,243,255,255,204,204,255,153,153,187,85,36,231,231,224,147,255,224,144,147,128,144,147,224,255,231,231
.db 255,231,255,231,7,147,19,3,16,0,19,0,16,0,147,0,255,147,7,7,255,147,0,231,255,0,255,240,15,0,145,204,204,201,249,129,153,196,225
.db 156,156,231,129,129,252,204,255,192,231,249,39,231,255,196,201,255,255,207,201,243,195,255

do_line4:	
	MOV  	DPL,_vpl
	MOV  	DPH,_vph
repeat_char4:
	MOVX 	A,@DPTR     
	INC     DPTR  	    
	MOVC	A,@A+PC     
	MOV		SBUF,A
	NOP
	NOP    
	DJNZ	_lcount,repeat_char4
	SETB	P1.1	
  	RET	
charline4:
.db 255,126,0,128,128,128,193,193,60,255,255,255,255,255,57,24,128,128,231,153,33,201,255,231,231,231,128,128,128,128,128,128,255
.db 243,255,201,225,231,145,255,249,231,0,192,255,192,255,243,132,243,227,227,204,207,224,231,225,193,255,255,252,255,207,231,132,204
.db 193,252,153,225,225,252,192,243,207,225,249,128,132,156,193,204,193,241,243,204,204,148,227,225,231,249,231,231,156,255,255,207
.db 193,204,193,204,240,204,145,243,207,201,243,128,204,204,153,204,145,252,243,204,204,148,201,204,230,248,255,199,255,201,204,204
.db 204,159,207,207,207,252,153,204,204,243,231,243,156,225,249,207,128,225,225,225,204,204,204,153,204,252,240,192,160,195,207,243
.db 225,204,224,200,131,227,249,192,192,132,36,231,204,51,238,170,136,231,231,231,147,255,231,159,147,159,159,147,231,255,231,231,255
.db 231,255,231,231,147,243,243,255,255,243,255,255,255,147,255,255,147,231,231,255,147,231,231,255,0,255,240,15,0,196,224,252,201,243
.db 228,153,231,204,128,156,193,36,36,224,204,192,243,243,243,231,231,192,255,227,231,255,207,201,249,195,255

do_line5:	
	MOV  	DPL,_vpl
	MOV  	DPH,_vph
repeat_char5:
	MOVX 	A,@DPTR     
	INC     DPTR  	    
	MOVC	A,@A+PC     
	MOV		SBUF,A
	NOP
	NOP
	DJNZ	_lcount,repeat_char5
	SETB	P1.1	
  	RET	
charline5:
.db  255, 66, 60, 193, 193, 128, 128, 128, 60, 255, 255, 255, 255, 255, 57, 24, 224, 131, 231, 153, 39, 201, 129, 129, 231, 129 
.db  207, 249, 249, 249, 249, 249, 255, 243, 255, 128, 207, 243, 196, 255, 249, 231, 195, 243, 255, 255, 255, 249, 144, 243, 249, 207
.db  128, 207, 204, 243, 204, 207, 255, 255, 249, 255, 231, 243, 132, 192, 153, 252, 153, 233, 233, 140, 204, 243, 204, 201, 185, 148
.db  140, 156, 249, 196, 201, 199, 243, 204, 204, 128, 227, 243, 179, 249, 207, 231, 255, 255, 255, 193, 153, 252, 204, 192, 249, 204
.db  153, 243, 207, 225, 243, 128, 204, 204, 153, 204, 153, 225, 243, 204, 204, 128, 227, 204, 243, 243, 231, 243, 255, 156, 225, 204
.db  192, 131, 193, 193, 193, 252, 129, 192, 192, 243, 231, 243, 128, 204, 225, 1, 204, 204, 204, 204, 204, 204, 204, 153, 204, 252
.db  249, 243, 156, 231, 193, 243, 204, 204, 204, 192, 255, 255, 252, 252, 207, 51, 19, 231, 153, 153, 187, 85, 36, 231, 224, 224, 144
.db  128, 224, 144, 147, 144, 128, 128, 224, 224, 7, 0, 0, 7, 0, 0, 7, 19, 3, 19, 0, 16, 19, 0, 16, 0, 0, 0, 0, 3, 7, 7, 3, 0, 0, 224
.db  7, 0, 0, 240, 15, 255, 236, 204, 252, 201, 249, 228, 153, 231, 204, 156, 201, 204, 36, 36, 252, 204, 255, 243, 249, 231, 231, 231
.db  255, 145, 255, 231, 231, 200, 201, 225, 195, 255

do_line6:	
	MOV  	DPL,_vpl
	MOV  	DPH,_vph
repeat_char6:
	MOVX 	A,@DPTR     
	INC     DPTR  	    
	MOVC	A,@A+PC     
	MOV		SBUF,A
	NOP
	NOP    
	DJNZ	_lcount,repeat_char6
	SETB	P1.1	
  	RET	
charline6:
.db  255, 102, 24, 227, 227, 193, 193, 193, 24, 255, 255, 255, 255, 255, 25, 195, 248, 143, 129, 255, 39, 227, 129, 195, 231, 195
.db  231, 243, 243, 243, 243, 243, 255, 255, 255, 201, 224, 153, 204, 255, 243, 243, 153, 243, 243, 255, 243, 252, 152, 243, 204, 204
.db  207, 204, 204, 243, 204, 231, 243, 243, 243, 192, 243, 255, 252, 204, 153, 153, 201, 185, 249, 153, 204, 243, 204, 153, 153, 156
.db  156, 201, 249, 225, 153, 204, 243, 204, 225, 136, 201, 243, 153, 249, 159, 231, 255, 255, 255, 204, 153, 204, 204, 252, 249, 193
.db  153, 243, 204, 201, 243, 148, 204, 204, 193, 193, 249, 207, 211, 204, 225, 128, 201, 193, 217, 243, 231, 243, 255, 156, 231, 204
.db  252, 153, 204, 204, 204, 225, 249, 252, 252, 243, 231, 243, 156, 192, 249, 204, 204, 204, 204, 204, 204, 204, 193, 195, 204, 129
.db  152, 192, 12, 231, 204, 243, 204, 204, 204, 196, 129, 193, 204, 252, 207, 153, 9, 231, 51, 204, 238, 170, 17, 231, 231, 231, 147
.db  147, 231, 147, 147, 147, 255, 255, 255, 231, 255, 255, 231, 231, 255, 231, 231, 147, 255, 147, 255, 147, 147, 255, 147, 255, 255
.db  231, 147, 255, 255, 231, 147, 147, 231, 255, 231, 0, 0, 240, 15, 255, 196, 224, 252, 201, 204, 228, 193, 231, 225, 201, 201, 204
.db  129, 129, 249, 204, 192, 255, 255, 255, 231, 228, 243, 196, 255, 255, 255, 201, 255, 255, 195, 255

do_line7:	
	MOV  	DPL,_vpl
	MOV  	DPH,_vph
repeat_char7:
	MOVX 	A,@DPTR     
	INC     DPTR  	    
	MOVC	A,@A+PC     
	MOV		SBUF,A
	NOP
	NOP	    
	DJNZ	_lcount,repeat_char7
	SETB	P1.1	
  	RET	
charline7:
.db 255, 126, 0, 247, 247, 227, 227, 227, 0, 255, 255, 255, 255, 255, 152, 165, 254, 191, 195, 153, 39, 204, 129, 231, 231, 231
.db 255, 255, 255, 255, 255, 255, 255, 243, 255, 201, 243, 156, 145, 255, 231, 249, 255, 255, 243, 255, 243, 254, 193, 192, 192, 225
.db 135, 225, 225, 243, 225, 241, 243, 243, 231, 255, 249, 243, 225, 204, 192, 195, 224, 128, 240, 131, 204, 225, 225, 152, 128, 156
.db 156, 227, 240, 199, 152, 225, 225, 192, 243, 156, 156, 225, 128, 225, 191, 225, 255, 255, 255, 145, 196, 225, 145, 225, 240, 207
.db 152, 225, 204, 152, 225, 156, 204, 225, 249, 207, 240, 224, 231, 145, 243, 201, 156, 207, 192, 199, 231, 248, 255, 128, 207, 129
.db 225, 3, 129, 129, 129, 207, 195, 225, 225, 225, 195, 225, 156, 204, 192, 1, 140, 225, 225, 225, 129, 129, 207, 231, 225, 231, 192
.db 243, 156, 228, 129, 225, 225, 129, 204, 204, 255, 255, 225, 255, 255, 204, 12, 231, 255, 255, 187, 85, 36, 231, 231, 231, 147
.db 147, 231, 147, 147, 147, 255, 255, 255, 231, 255, 255, 231, 231, 255, 231, 231, 147, 255, 147, 255, 147, 147, 255, 147, 255, 255
.db 231, 147, 255, 255, 231, 147, 147, 231, 255, 231, 0, 0, 240, 15, 255, 145, 252, 252, 201, 192, 241, 249, 231, 243, 227, 136, 225
.db 255, 249, 227, 204, 255, 192, 192, 192, 231, 228, 243, 255, 255, 255, 255, 195, 255, 255, 255, 255

do_line8:	
	MOV  	DPL,_vpl
	MOV  	DPH,_vph
repeat_char8:
	MOVX 	A,@DPTR     
	INC     DPTR  	    
	MOVC	A,@A+PC     
	MOV		SBUF,A
	NOP
	NOP	    
	DJNZ	_lcount,repeat_char8		
	LJMP 	LINE8_END

charline8:
.db 255, 129, 129, 255, 255, 193, 193, 193, 0, 255, 255, 255, 255, 255, 252, 102, 255, 255, 231, 255, 255, 225, 255, 0, 255, 255
.db 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 249, 255, 255, 255, 255, 255, 255, 255
.db 255, 255, 255, 255, 255, 255, 255, 249, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255
.db 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 0, 255, 255, 255, 255, 255, 255, 255, 224
.db 255, 255, 225, 255, 255, 255, 255, 255, 240, 135, 255, 255, 255, 255, 255, 255, 255, 224, 255, 255, 255, 255, 255, 255, 225, 255
.db 255, 255, 255, 255, 255, 227, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 224, 255, 255, 231
.db 255, 243, 28, 241, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 15, 63, 255, 255, 255, 238, 170, 136, 231, 231, 231
.db 147, 147, 231, 147, 147, 147, 255, 255, 255, 231, 255, 255, 231, 231, 255, 231, 231, 147, 255, 147, 255, 147, 147, 255, 147, 255
.db 255, 231, 147, 255, 255, 231, 147, 147, 231, 255, 231, 0, 0, 240, 15, 255, 255, 252, 255, 255, 255, 255, 252, 255, 192, 255, 255
.db 255, 255, 252, 255, 255, 255, 255, 255, 255, 231, 241, 255, 255, 255, 255, 255, 199, 255, 255, 255, 255
LINE8_END:
	SETB	P1.1
	JNB		_vpixel_end,SKIP_REPEAT
	MOV  	_vpl,DPL
	MOV  	_vph,DPH	
SKIP_REPEAT:
  	RET	
	
__endasm;
}
